IF NOT OBJECT_ID('[dbo].[tsp_ClearSysPrcPersistentStore]') IS NULL
BEGIN
	DROP PROCEDURE [dbo].[tsp_ClearSysPrcPersistentStore]
END;
GO

CREATE PROCEDURE [tsp_ClearSysPrcPersistentStore]
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @startDate DATETIME;
	SET @startDate = GETDATE();

	PRINT 'Database: ' + DB_NAME();

	DECLARE @allRowsCount bigint = (SELECT COUNT_BIG(*) FROM [dbo].[SysPrcPersistentStore])
	PRINT SPACE(4) + 'Number of records = ' + CAST(@allRowsCount AS nvarchar(50))

	CREATE TABLE #Source  (Id uniqueidentifier);
	CREATE TABLE #DeleteIds  (Id uniqueidentifier);

	WITH [GrouppedRecords] ([Id], [RowNumber])
	AS (
		SELECT
		t1.[Id]
		,ROW_NUMBER() OVER (PARTITION BY [GroupKey], [Key] ORDER BY t1.[CreatedOn] DESC) AS RowNumber
		FROM [dbo].[SysPrcPersistentStore] t1
	)
	INSERT INTO #Source([Id])
	SELECT [Id] FROM [GrouppedRecords] WHERE [RowNumber] > 1

	DECLARE @count int = 0 ;
	DECLARE @step int = 1000 ;
	DECLARE @rowsleft int = 0 ;

	DELETE TOP (@step)
	FROM #Source
	OUTPUT deleted.[Id] INTO #DeleteIds ([Id]);

	SELECT @count = COUNT(*) FROM #DeleteIds;

	WHILE @count > 0
		BEGIN
			PRINT SPACE(4) + 'Step Begin: ' + CONVERT(nvarchar(30), GETDATE(), 114);
			BEGIN TRAN
				DELETE FROM [dbo].[SysPrcPersistentStore] 
				FROM [dbo].[SysPrcPersistentStore] AS t1
				INNER JOIN #DeleteIds t2 on t2.Id = t1.Id
				           
				TRUNCATE TABLE #DeleteIds;
      
				SELECT @rowsleft = COUNT(*) FROM #Source

				DELETE TOP (@step)
				FROM #Source
				OUTPUT deleted.[Id] INTO #DeleteIds ([Id]);
      
				SELECT @count = COUNT(*) FROM #DeleteIds;
			COMMIT TRAN
              
			PRINT SPACE(4) + 'Step ready: ' + CONVERT(nvarchar(30), GETDATE(), 114) + N' Rows Left: ' + CAST(@rowsleft as nvarchar(255))
			--WAITFOR DELAY '000:00:02'
		END;
	
	DROP TABLE #DeleteIds;
	DROP TABLE #Source;

	PRINT '';
	SET @allRowsCount = @allRowsCount - (SELECT COUNT_BIG(*) FROM [dbo].[SysPrcPersistentStore])
	PRINT SPACE(4) + 'Number of records processed = ' + CAST(@allRowsCount AS nvarchar(50))
	PRINT '---------------------------------------------------------------';
	PRINT 'Execution time: ' + CONVERT(nvarchar(30), GETDATE() - @startDate, 114);
	PRINT '';
END;
GO

EXEC [tsp_ClearSysPrcPersistentStore];