SET NOCOUNT ON;

DECLARE @drop NVARCHAR(MAX) = N'';
SELECT @drop += N'
ALTER TABLE ' + QUOTENAME(cs.name) + '.' + QUOTENAME(ct.name)
	+ ' DROP CONSTRAINT ' + QUOTENAME(fk.name) + ';'
FROM sys.foreign_keys AS fk
INNER JOIN sys.tables AS ct ON fk.parent_object_id = ct.[object_id]
INNER JOIN sys.schemas AS cs ON ct.[schema_id] = cs.[schema_id]
WHERE fk.name IN (
	'FKRD77Ycitqn49r1ecdgOib3l0', 'FKOj6EFZKQgbWFSlzcXwLerIEtD7k',
	'FKmYgNj522j9H4JyGrpK5zE4CnfQ0', 'FKsbMfIJDfqUhedocgiL3mxbOhs',
	'FK6bJMcMlwYQLCNgFXtapnmWGiWA', 'FKYgqaBK81UhrkHG398ZuPu4DaU', 'FKH3ENFrj2c6LOYgG5lBhyGaRU3A'
)

EXEC sp_executesql @drop;
GO

IF OBJECT_ID(N'[dbo].[tsp_ProcessLogArchiving]', N'P') IS NOT NULL
	DROP PROCEDURE [dbo].[tsp_ProcessLogArchiving];
GO

CREATE PROCEDURE [dbo].[tsp_ProcessLogArchiving]
	@statusId UNIQUEIDENTIFIER,
	@archivingPeriod SMALLINT,
	@chunkSize INT
AS
BEGIN
	DECLARE @processLogIds TABLE (Id UNIQUEIDENTIFIER);

	INSERT INTO @processLogIds
	SELECT TOP(@chunkSize) [Id]
		FROM [dbo].[SysProcessLog]
			WHERE [StatusId] = @statusId
				AND [CompleteDate] <= DATEADD(DAY, @archivingPeriod, GETDATE());

	SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;
	BEGIN TRAN;

		DELETE FROM [dbo].[SysProcessEntity]
		OUTPUT
			deleted.[CreatedOn], deleted.[CreatedById], deleted.[ModifiedOn],
			deleted.[ModifiedById], deleted.[SysProcessId], deleted.[EntityId],
			deleted.[EntityDisplayValue], deleted.[ProcessListeners],
			deleted.[EntitySchemaUId]
		INTO [dbo].[SysPrcEntityHistory] ([CreatedOn], [CreatedById], [ModifiedOn],
			[ModifiedById], [SysProcessId], [EntityId],
			[EntityDisplayValue], [ProcessListeners], [EntitySchemaUId])
		WHERE [SysProcessId] IN (SELECT Id FROM @processLogIds)

		DELETE FROM [dbo].[SysProcessElementLog]
		OUTPUT
			deleted.[CreatedOn], deleted.[CreatedById], deleted.[ModifiedOn],
			deleted.[ModifiedById], deleted.[SysProcessId], deleted.[Caption],
			deleted.[CompleteDate], deleted.[ProcessListeners],
			deleted.[SchemaElementUId], deleted.[StartDate], deleted.[StatusId],
			deleted.[Type], deleted.[OwnerId], deleted.[ErrorDescription],
			deleted.[DurationInMinutes], deleted.[DurationInHours], deleted.[DurationInDays]
		INTO [dbo].[SysPrcElHistoryLog] ([CreatedOn], [CreatedById], [ModifiedOn],
			[ModifiedById], [SysProcessId], [Caption], [CompleteDate], [ProcessListeners],
			[SchemaElementUId], [StartDate], [StatusId], [Type], [OwnerId], [ErrorDescription],
			[DurationInMinutes], [DurationInHours], [DurationInDays])
		WHERE [SysProcessId] IN (SELECT Id FROM @processLogIds)

		DELETE FROM [dbo].[SysProcessLog]
		OUTPUT
			deleted.[Id], deleted.[CreatedOn], deleted.[CreatedById], deleted.[ModifiedOn],
			deleted.[ModifiedById], deleted.[ProcessListeners],
			deleted.[Name], deleted.[StartDate], deleted.[CompleteDate], deleted.[SysSchemaId],
			deleted.[ParentId], deleted.[StatusId], deleted.[OwnerId], deleted.[SysWorkspaceId],
			deleted.[DurationInMinutes], deleted.[DurationInHours], deleted.[DurationInDays],
			deleted.[PackageName], deleted.[Version]
		INTO [dbo].[SysPrcHistoryLog] ([Id], [CreatedOn], [CreatedById], [ModifiedOn],
			[ModifiedById], [ProcessListeners], [Name], [StartDate], [CompleteDate],
			[SysSchemaId], [ParentId], [StatusId], [OwnerId], [SysWorkspaceId],
			[DurationInMinutes], [DurationInHours], [DurationInDays], [PackageName], [Version])
		WHERE [Id] IN (SELECT Id FROM @processLogIds)

	COMMIT TRAN;

END;
GO

DECLARE @archivingPeriod SMALLINT = ISNULL(-(
		SELECT sv.[IntegerValue]
		FROM [dbo].[SysSettings] s
		JOIN [dbo].[SysSettingsValue] sv ON sv.[SysSettingsId] = s.[Id]
		WHERE s.[Code] = 'ProcessLogArchivingPeriod'
	), -30);
DECLARE @statusId UNIQUEIDENTIFIER = '815C9586-B6E2-DF11-971B-001D60E938C6';
DECLARE @chunkSize INT = 500000;

DECLARE @processLogRecordsCount INT = (
	SELECT COUNT(*) FROM [dbo].[SysProcessLog]
	WHERE [StatusId] = @statusId
		AND [CompleteDate] <= DATEADD(DAY, @archivingPeriod, GETDATE()));
WHILE @processLogRecordsCount > 0
BEGIN
	EXEC [dbo].[tsp_ProcessLogArchiving] @statusId, @archivingPeriod, @chunkSize;
	SET @processLogRecordsCount = @processLogRecordsCount - @chunkSize;
END;
GO

DROP PROCEDURE [dbo].[tsp_ProcessLogArchiving];
GO

DECLARE @create NVARCHAR(MAX) = N'';
SELECT @create += N'
ALTER TABLE '
	+ QUOTENAME(cs.name) + '.' + QUOTENAME(ct.name)
	+ ' ADD CONSTRAINT ' + QUOTENAME(fk.name)
	+ ' FOREIGN KEY (' + STUFF((SELECT ',' + QUOTENAME(c.name)
	FROM sys.columns AS c
	INNER JOIN sys.foreign_key_columns AS fkc
		ON fkc.parent_column_id = c.column_id
		AND fkc.parent_object_id = c.[object_id]
	WHERE fkc.constraint_object_id = fk.[object_id]
	ORDER BY fkc.constraint_column_id
	FOR XML PATH(N''), TYPE).value(N'.[1]', N'nvarchar(max)'), 1, 1, N'')
		+ ') REFERENCES ' + QUOTENAME(rs.name) + '.' + QUOTENAME(rt.name)
		+ '(' + STUFF((SELECT ',' + QUOTENAME(c.name)
	FROM sys.columns AS c
	INNER JOIN sys.foreign_key_columns AS fkc
		ON fkc.referenced_column_id = c.column_id
		AND fkc.referenced_object_id = c.[object_id]
	WHERE fkc.constraint_object_id = fk.[object_id]
	ORDER BY fkc.constraint_column_id
	FOR XML PATH(N''), TYPE).value(N'.[1]', N'nvarchar(max)'), 1, 1, N'') + ');'
FROM sys.foreign_keys AS fk
INNER JOIN sys.tables AS rt
	ON fk.referenced_object_id = rt.[object_id]
INNER JOIN sys.schemas AS rs ON rt.[schema_id] = rs.[schema_id]
INNER JOIN sys.tables AS ct
	ON fk.parent_object_id = ct.[object_id]
INNER JOIN sys.schemas AS cs ON ct.[schema_id] = cs.[schema_id]
WHERE fk.name IN (
	'FKRD77Ycitqn49r1ecdgOib3l0', 'FKOj6EFZKQgbWFSlzcXwLerIEtD7k',
	'FKmYgNj522j9H4JyGrpK5zE4CnfQ0', 'FKsbMfIJDfqUhedocgiL3mxbOhs',
	'FK6bJMcMlwYQLCNgFXtapnmWGiWA', 'FKYgqaBK81UhrkHG398ZuPu4DaU', 'FKH3ENFrj2c6LOYgG5lBhyGaRU3A'
)

EXEC sp_executesql @create;
GO
